package jrummikub.view.impl;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Insets;
import java.awt.RenderingHints;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.util.Collections;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JPanel;

import jrummikub.model.Position;
import jrummikub.model.Stone;
import jrummikub.util.Event1;
import jrummikub.util.IEvent1;
import jrummikub.view.IBoard;

@SuppressWarnings("serial")
public class Board extends JPanel implements IBoard {
  private final static ImageIcon background = new ImageIcon(Board.class.getResource("/jrummikub/resource/wood.png"));

  private Map<Stone, Position> stones = Collections.emptyMap();
  private StonePainter stonePainter = new StonePainter(StonePainter.BOARD_SCALE);
  
  private Event1<Position> clickEvent = new Event1<Position>();

  Board() {
    super(true); // set double buffered
    
    setBorder(new CustomBorder(Color.DARK_GRAY, 0, 1, 0, 1));

    addMouseListener(new MouseAdapter() {
      @Override
      public void mouseClicked(MouseEvent e) {
        Insets insets = getInsets();

        clickEvent.fire(stonePainter.calculatePosition(e.getX() - insets.left,
            e.getY() - insets.top));
      }
    });
  }

  @Override
  public IEvent1<Position> getClickEvent() {
    return clickEvent;
  }
  
  @Override
  protected void paintComponent(Graphics g1) {
    Insets insets = getInsets();
    int x = insets.left, y = insets.top, width = getWidth()-insets.left-insets.right, height = getHeight()-insets.top-insets.bottom;
    Graphics2D g = (Graphics2D)g1.create(x, y, width, height);
    
    for(int xpos = 0; xpos < width; xpos += background.getIconWidth()) {
      background.paintIcon(this, g, xpos, 0);
    }
    for(int xpos = -32; xpos < width; xpos += background.getIconWidth()) {
      background.paintIcon(this, g, xpos, 75);
    }
    
    g.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
        RenderingHints.VALUE_ANTIALIAS_ON);

    for (Map.Entry<Stone, Position> stone : stones.entrySet()) {
      stonePainter.paintStone(g, stone.getKey(), stone.getValue());
    }
  }

  @Override
  public void setStones(Map<Stone, Position> stones) {
    this.stones = stones;
  }
}
